@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/type.c,v 1.62 2004/04/16 05:26:45 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/type.c,v 1.63 2004/05/26 08:59:23 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -69,7 +69,8 @@ TypeArrayMark (void *object)
     case DimStorageGlobal:
 	MemReference (ta->u.global);
 	break;
-    case DimStorageLocal:
+    case DimStorageStatic:
+    case DimStorageAuto:
 	MemReference (ta->u.frame.code);
 	break;
     }
